<?php

namespace Drupal\foundation_anchor_menu_block\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Block\BlockPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;

/**
 * Provides a dynamic 'Foundation Anchor Menu' Block.
 *
 * @Block(
 *   id = "foundation_anchor_menu_block_block",
 *   admin_label = @Translation("Foundation Anchor Menu Block (Dynamic by JavaScript)"),
 *   admin_description = @Translation("Only visible if at least one anchor menu item exists on the current page. Content created at runtime by JavaScript."),
 *   category = @Translation("Navigation"),
 * )
 */
class FoundationAnchorMenuBlock extends BlockBase implements BlockPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function build() {
    return [
      '#theme' => 'famb_anchor_menu',
      '#menu_orientation' => $this->configuration['menu_orientation'],
      '#menu_horizontal_breakpoint' => $this->configuration['menu_horizontal_breakpoint'],
      '#list_classes_additional' => array_unique(explode(' ', $this->configuration['list_classes_additional'])),
      '#attached' => [
        'library' => [
          'foundation_anchor_menu_block/famb-anchor-menu',
        ],
        'drupalSettings' => [
          'foundation_anchor_menu_block' => [
            'menu_orientation' => $this->configuration['menu_orientation'],
            'menu_horizontal_breakpoint' => $this->configuration['menu_horizontal_breakpoint'],
            'list_classes_additional' => $this->configuration['list_classes_additional'],
            'animationDuration' => $this->configuration['animationDuration'],
            'animationEasing' => $this->configuration['animationEasing'],
            'threshold' => $this->configuration['threshold'],
            'activeClass' => $this->configuration['activeClass'],
            'deeplinking' => $this->configuration['deeplinking'],
            'updateHistory' => $this->configuration['updateHistory'],
            'offsetTopBarSelector' => $this->configuration['offsetTopBarSelector'],
            'offset' => $this->configuration['offset'],
          ]
        ]
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'menu_orientation' => 'horizontal',
      'menu_horizontal_breakpoint' => 'medium',
      'list_classes_additional' => '',
      'animationDuration' => '500',
      'animationEasing' => 'linear',
      'threshold' => 50,
      'activeClass' => 'is-active',
      'deeplinking' => false,
      'updateHistory' => false,
      'offsetTopBarSelector' => '.top-bar:first',
      'offset' => 0,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    // Retrieve existing configuration for this block.
    $config = $this->getConfiguration();
    $form['menu_orientation'] = [
      '#type' => 'select',
      '#title' => t('Menu orientation'),
      '#options' => [
        'vertical' => $this->t('Vertical'),
        'horizontal' => $this->t('Horizontal'),
      ],
      '#required' => TRUE,
      '#default_value' => isset($config['menu_orientation']) ? $config['menu_orientation'] : 'horizontal',
      '#description' => $this->t('See <a href="@foundation_docs_vertical_menu_href" target="_blank">Foundation Docs: Vertical Menu</a>', ['@foundation_docs_vertical_menu_href' => 'https://get.foundation/sites/docs/menu.html#vertical-menu']),
    ];
    $form['menu_horizontal_breakpoint'] = [
      '#type' => 'select',
      '#title' => t('Menu horizontal breakpoint'),
      '#options' => [
        'small' => $this->t('Small'),
        'medium' => $this->t('Medium') . ' (' . $this->t('Default') . ')',
        'large' => $this->t('Large'),
      ],
      '#required' => TRUE,
      '#default_value' => isset($config['menu_horizontal_breakpoint']) ? $config['menu_horizontal_breakpoint'] : 'medium',
      '#description' => $this->t('If the screen is smaller than this breakpoint, switches a horizontal menu to vertical.'),
    ];
    $form['list_classes_additional'] = [
      '#type' => 'textfield',
      '#title' => t('Additional list classes'),
      '#required' => FALSE,
      '#default_value' => isset($config['list_classes_additional']) ? $config['list_classes_additional'] : '',
      '#description' => $this->t('Adds further custom classes to the list (&lt;ul&gt;) element. Separated by space.'),
    ];

    // Magellan settings:
    // TODO - Put in a field group.
    $form['animationDuration'] = [
      '#title' => t('Animation Duration'),
      '#type' => 'number',
      '#step' => 1,
      '#min' => 0,
      '#size' => 5,
      '#required' => TRUE,
      '#default_value' => isset($config['animationDuration']) ? $config['animationDuration'] : 500,
      '#description' => $this->t('Amount of time, in ms, the animated scrolling should take between locations. Default: 500'),
    ];
    $form['animationEasing'] = [
      '#title' => t('Animation Easing'),
      '#type' => 'select',
      '#options' => [
        'linear' => $this->t('Linear'),
        'swing' => $this->t('Swing'),
      ],
      '#required' => TRUE,
      '#default_value' => isset($config['animationEasing']) ? $config['animationEasing'] : 'linear',
      '#description' => $this->t('Animation style to use when scrolling between locations. Can be `swing` or `linear`.  Default: linear'),
    ];
    $form['threshold'] = [
      '#title' => t('Threshold'),
      '#type' => 'number',
      '#step' => 1,
      '#min' => 0,
      '#size' => 5,
      '#required' => TRUE,
      '#default_value' => isset($config['threshold']) ? $config['threshold'] : 50,
      '#description' => $this->t('Number of pixels to use as a marker for location changes. Default: 50'),
    ];
    $form['activeClass'] = [
      '#type' => 'textfield',
      '#title' => t('Active Class'),
      '#required' => TRUE,
      '#default_value' => isset($config['activeClass']) ? $config['activeClass'] : 'is-active',
      '#description' => $this->t('Class applied to the active locations link on the magellan container. Default: is-active'),
    ];
    $form['deeplinking'] = [
      '#type' => 'checkbox',
      '#title' => t('Deeplinking'),
      '#required' => FALSE,
      '#default_value' => isset($config['deeplinking']) ? $config['deeplinking'] : FALSE,
      '#description' => $this->t('Allows the script to manipulate the url of the current page, and if supported, alter the history.  Default: FALSE'),
    ];
    $form['updateHistory'] = [
      '#type' => 'checkbox',
      '#title' => t('Update History'),
      '#required' => FALSE,
      '#default_value' => isset($config['updateHistory']) ? $config['updateHistory'] : FALSE,
      '#description' => $this->t('Update the browser history with the active link, if deep linking is enabled. Default: FALSE'),
    ];
    $form['offsetTopBarSelector'] = [
      '#type' => 'textfield',
      '#title' => t('Offset (Topbar) Selector'),
      '#required' => FALSE,
      '#default_value' => isset($config['offsetTopBarSelector']) ? $config['offsetTopBarSelector'] : '.top-bar:first',
      '#description' => $this->t('jQuery Top bar selector. If given used to calculate the offset automatically. Leave emtpy to only use a manual offset value. Default: .top-bar:first'),
    ];
    $form['offset'] = [
      '#type' => 'number',
      '#step' => 1,
      '#title' => t('Offset (additional)'),
      '#required' => FALSE,
      '#default_value' => isset($config['offset']) ? $config['offset'] : 0,
      '#description' => $this->t('Number of pixels to offset the scroll of the page on item click if using a sticky nav bar. Summed up with offsetTopBarSelector.outerHeight().'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    // Save our custom settings when the form is submitted.
    $this->setConfigurationValue('menu_orientation', $form_state->getValue('menu_orientation'));
    $this->setConfigurationValue('menu_horizontal_breakpoint', $form_state->getValue('menu_horizontal_breakpoint'));
    $this->setConfigurationValue('list_classes_additional', Html::escape(trim($form_state->getValue('list_classes_additional'))));
    // Magellan settings:
    $this->setConfigurationValue('animationDuration', $form_state->getValue('animationDuration'));
    $this->setConfigurationValue('animationEasing', $form_state->getValue('animationEasing'));
    $this->setConfigurationValue('threshold', $form_state->getValue('threshold'));
    $this->setConfigurationValue('activeClass', $form_state->getValue('activeClass'));
    $this->setConfigurationValue('deeplinking', $form_state->getValue('deeplinking'));
    $this->setConfigurationValue('updateHistory', $form_state->getValue('updateHistory'));
    $this->setConfigurationValue('offsetTopBarSelector', $form_state->getValue('offsetTopBarSelector'));
    $this->setConfigurationValue('offset', $form_state->getValue('offset'));
  }

  /**
   * {@inheritdoc}
   */
  public function blockValidate($form, FormStateInterface $form_state) {
    $menu_orientation = $form_state->getValue('menu_orientation');
    if (!in_array($menu_orientation, ['vertical', 'horizontal'])) {
      $form_state->setErrorByName('menu_orientation', $this->t('Must be @vertical or @horizontal', ['@vertical' => $this->t('Vertical'), '@horizontal' => $this->t('Horizontal')]));
    }
    $menu_horizontal_breakpoint = $form_state->getValue('menu_horizontal_breakpoint');
    if (!in_array($menu_horizontal_breakpoint, ['small', 'medium', 'large'])) {
      $form_state->setErrorByName('menu_horizontal_breakpoint', $this->t('Must be small, medium or large.'));
    }
  }

}
